/*
 *  PeaGnut - Command-line Gnutella client for Unix
 *  Copyright (C) 2000  Ciaran Lynch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <netdb.h>
#include <signal.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#include "connection.h"
#include "log.h"
#include "completeio.h"
#include "conlist.h"
#include "connectionlist.h"

#define INCOMING_CONNECTIONS_MAX 10

int connection_handle(int socket)
{
	struct sockaddr_in receive_address;
	int receive_socket,receive_length=sizeof(receive_address);
	char *host_addr,host_port[10]; /* XXX Fixed-size buffer, but 16-bit integer can't overflow it ... */
	
	if((receive_socket=accept(socket,(struct sockaddr *)&receive_address,&receive_length))==-1) 
	{
		log(LOG_ERR,"Unable to handle connection : Error accepting on socket : %m");
		return 1;
	}

	if((fcntl(receive_socket,F_SETFL,O_NONBLOCK))==-1)
	{
		log(LOG_ERR,"Unable to handle connection : Error setting receive socket flags : %m");
		if(shutdown(receive_socket,2)==-1)
			log(LOG_ERR,"Unable to shutdown receive socket : %m");
		if(close(receive_socket)==-1)
			log(LOG_ERR,"Unable to close receive socket : %m");
		return 1;
	}
	
	if(connection_list_size()>=INCOMING_CONNECTIONS_MAX)
	{
		/* Refuse Connection */
		if(shutdown(receive_socket,2)==-1)
		{
			log(LOG_ERR,"Unable to handle connection : Error Shutting down receive socket : %m");
			if(close(receive_socket)==-1)
				log(LOG_ERR,"Unable to close receive socket : %m");
			return 1;
		}
		return 0;
	}
	
	if((host_addr=strdup(inet_ntoa(receive_address.sin_addr)))==NULL)
	{
		log(LOG_ERR,"Unable to handle connection : Error converting receive socket address : %m");
		if(shutdown(receive_socket,2)==-1)
			log(LOG_ERR,"Unable to shutdown receive socket : %m");
		if(close(receive_socket)==-1)
			log(LOG_ERR,"Unable to close receive socket : %m");
		return 1;
	}
	sprintf(host_port,"%d",ntohs(receive_address.sin_port));
	/* Add connection to list */
	
	if(connection_list_add(host_addr,host_port,CONNECTION_REQUESTED_IN,receive_socket))
	{
		log(LOG_ERR,"Unable to handle connection from %s:%s : Error adding connection to list",host_addr,host_port);
		if(shutdown(receive_socket,2)==-1)
			log(LOG_ERR,"Unable to shutdown receive socket : %m");
		if(close(receive_socket)==-1)
			log(LOG_ERR,"Unable to close receive socket : %m");
		free(host_addr);
		return 1;
	}
	log(LOG_INFO,"Incoming connection initiated : %s:%s",host_addr,host_port);
	free(host_addr);
	
	return 0;
}
