/*
 *  PeaGnut - Command-line Gnutella client for Unix
 *  Copyright (C) 2000  Ciaran Lynch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>

#include "log.h"
#include "connectionlist.h"
#include "list.h"
#include "conlist.h"

list_type *connection_list_internal=NULL;
int connection_list_size_internal=0;

int connection_list_add(char *address,char *port,int status,int con_socket)
{
	return list_add(address,port,status,con_socket,&connection_list_internal,&connection_list_size_internal);
}

int connection_list_remove(int index)
{
	return list_remove(index,&connection_list_internal,&connection_list_size_internal);
}

list_type *connection_list(int index)
{
	return list(index,&connection_list_internal,&connection_list_size_internal);
}

int connection_list_update_status(int index,int status)
{
	return list_update_status(index,status,&connection_list_internal,&connection_list_size_internal);
}

int connection_list_free(void)
{
	return list_free(&connection_list_internal,&connection_list_size_internal);
}

int connection_list_size(void)
{
	return connection_list_size_internal;
}

int connection_list_write_response(int index)
{
	int retval;
	retval=write(connection_list_internal[index].con_socket,"GNUTELLA OK\n\n",13);
	if(retval==-1)
	{
		log(LOG_ERR,"Unable to write to socket : %m");
		return -1;
	} else {
		return 1; /* Success */
	}
	return -1; /* Fallthrough = Error */
}

int connection_list_write_challenge(int index)
{
	int retval;
	retval=write(connection_list_internal[index].con_socket,"GNUTELLA CONNECT/0.4\n\n",22);
	if(retval==-1)
	{
		log(LOG_ERR,"Unable to write to socket : %m");
		return -1;
	} else {
		if(connection_list_update_status(index,CONNECTION_REQUESTED_OUT))
		{
			return -1;
		}
		log(LOG_INFO,"Connection requested to %s:%s",connection_list_internal[index].address,connection_list_internal[index].port);
		return 1; /* Success */
	}
	return -1; /* Fallthrough = Error */
}

int connection_list_check_challenge(int index)
{
	int retval;
	
	if(!connection_list_internal[index].response_buffer) connection_list_internal[index].response_buffer=malloc(14);
	retval=read(connection_list_internal[index].con_socket,(connection_list_internal[index].response_buffer)+(connection_list_internal[index].response_size),13-connection_list_internal[index].response_size);
	if(retval==-1)
	{
		/* Error */
		if((errno==EINTR)||(errno==EAGAIN)) return -2;
		log(LOG_ERR,"Error reading from socket : %m");
		return -1;
	} else if(retval==0) {
		/* Socket closed */
		if(connection_list_internal[index].response_buffer)
		{
			free(connection_list_internal[index].response_buffer);
			connection_list_internal[index].response_buffer=NULL;
		}	
		connection_list_internal[index].response_size=0;
		return 0;
	} else {
		/* Data ready */
		connection_list_internal[index].response_size+=retval;
		if(connection_list_internal[index].response_size==13)
		{
			/* Check ... */
			if(!memcmp(connection_list_internal[index].response_buffer,"GNUTELLA OK\n\n",13))
			{
				if(connection_list_update_status(index,CONNECTION_ESTABLISHED)==-1)
				{
					log(LOG_ERR,"Unable to process connection : Error updating connection status");
					retval=-1;
				} else {
					log(LOG_INFO,"Connection established to %s:%s",connection_list_internal[index].address,connection_list_internal[index].port);
					retval=1;
				}	
			} else  {
				connection_list_internal[index].response_buffer[13]='\0';
				log(LOG_INFO,"Incorrect response : %s",connection_list_internal[index].response_buffer);
				retval=0;
			}
			free(connection_list_internal[index].response_buffer);
			connection_list_internal[index].response_buffer=NULL;
			connection_list_internal[index].response_size=0;
			
			return retval;
		}
	}
	return -2; /* Not finished yet */
}

int connection_list_check_response(int index)
{
	int retval;
	
	if(!connection_list_internal[index].response_buffer) connection_list_internal[index].response_buffer=malloc(23);
	retval=read(connection_list_internal[index].con_socket,(connection_list_internal[index].response_buffer)+(connection_list_internal[index].response_size),22-connection_list_internal[index].response_size);
	if(retval==-1)
	{
		/* Error */
		if((errno==EINTR)||(errno==EAGAIN)) return -2;
		log(LOG_ERR,"Error reading from socket : %m");
		return -1;
	} else if(retval==0) {
		/* Socket closed */
		if(connection_list_internal[index].response_buffer)
		{
			free(connection_list_internal[index].response_buffer);
			connection_list_internal[index].response_buffer=NULL;
		}	
		connection_list_internal[index].response_size=0;
		return 0;
	} else {
		/* Data ready */
		connection_list_internal[index].response_size+=retval;
		log(LOG_INFO,"%d bytes read",connection_list_internal[index].response_size);
		if(connection_list_internal[index].response_size==22)
		{
			/* Check ... */
			if(!memcmp(connection_list_internal[index].response_buffer,"GNUTELLA CONNECT/0.4\n\n",22))
			{
				if(connection_list_update_status(index,CONNECTION_ESTABLISHED)==-1)
				{
					log(LOG_ERR,"Unable to process connection : Error updating connection status");
					retval=-1;
				}
				retval=1;	
			} else {
				connection_list_internal[index].response_buffer[22]='\0';
				log(LOG_ERR,"Invalid challenge : %s",connection_list_internal[index].response_buffer);
				retval=0;
			}
			free(connection_list_internal[index].response_buffer);
			connection_list_internal[index].response_buffer=NULL;
			connection_list_internal[index].response_size=0;
			
			return retval;
		}
	}
	return -2; /* Not finished yet */
}

