/*
 *  PeaGnut - Command-line Gnutella client for Unix
 *  Copyright (C) 2000  Ciaran Lynch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "list.h"
#include "log.h"

int list_add(char *address,char *port,int status,int con_socket,list_type **thelist,int *size)
{
	char *new_address,*new_port;
	new_address=strdup(address);
	if(new_address==NULL)
	{
		log(LOG_ERR,"Unable to add %s:%s to list : Out of memory",address,port);
		return 1;	
	}
	new_port=strdup(port);
	if(new_port==NULL)
	{
		log(LOG_ERR,"Unable to add %s:%s to list : Out of memory",address,port);
		return 1;
	}
	
	if(*size)
	{
		list_type *new_list=malloc(sizeof(list_type)*((*size)+1));
		if(new_list==NULL)
		{
			log(LOG_ERR,"Unable to add %s:%s to list : Out of memory",address,port);
			free(new_address);
			free(new_port);
			return 1;	
		}
		memcpy(new_list,*thelist,sizeof(list_type)*(*size));
		free(*thelist);
		*thelist=new_list;	
	} else {
		*thelist=malloc(sizeof(list_type));
		if((*thelist)==NULL)
		{
			log(LOG_ERR,"Unable to add %s:%s to list : Out of memory",address,port);
			free(new_address);
			free(new_port);
			return 1;	
		}
	}

	(*thelist)[*size].address=new_address;
	(*thelist)[*size].port=new_port;
	(*thelist)[*size].status=status;
	(*thelist)[*size].con_socket=con_socket;
	(*thelist)[*size].num_packets=0;
	(*thelist)[*size].packet=NULL;
	(*thelist)[*size].response_buffer=NULL;
	(*thelist)[(*size)++].response_size=0;

	return 0;
}

int list_remove(int index,list_type **thelist,int *size)
{
	if(index>=(*size)) return 1;

	if((*size)==1)
	{
		(*size)=0;
		free(thelist[0]->address);
		free(thelist[0]->port);
		free(*thelist);
		(*size)=0;
	} else {
		list_type *new_list=malloc(sizeof(list_type)*((*size)-1));
		int i;
		
		if(new_list==NULL)
		{
			log(LOG_ERR,"Unable to remove %s:%s from list : Out of memory",(*thelist)[index].address,(*thelist)[index].port);
			return 1;
		}
		
		for(i=0;i<(*size);i++)
		{
			if(i!=index) new_list[i<index?i:i-1]=(*thelist)[i];
		}
		free((*thelist)[index].address);
		free((*thelist)[index].port);

		free(*thelist);
		*thelist=new_list;
		(*size)--;
	}

	return 0;
}

list_type *list(int index,list_type **thelist,int *size)
{
	if(index>=(*size)) return NULL;
	return (*thelist)+index;
	/*
	else {
		list_type *return_list=malloc(sizeof(list_type));
		if(return_list==NULL)
		{
			log(LOG_ERR,"Unable to return list item %s:%s : Out of memory",(*thelist)[index].address,(*thelist)[index].port);
			return NULL;
		}	

		return_list->address=strdup((*thelist)[index].address);
		if(return_list->address==NULL)
		{
			log(LOG_ERR,"Unable to return list item %s:%s : Out of memory",(*thelist)[index].address,(*thelist)[index].port);
			free(return_list);
			return NULL;
		}
		return_list->port=strdup((*thelist)[index].port);
		if(return_list->port==NULL)
		{
			log(LOG_ERR,"Unable to return list item %s:%s : Out of memory",(*thelist)[index].address,(*thelist)[index].port);
			free(return_list->address);
			free(return_list);
			return NULL;
		}
		return_list->status=(*thelist)[index].status;
		return_list->con_socket=(*thelist)[index].con_socket;
		return_list->
	
		return return_list;
	}*/
}

int list_free(list_type **thelist,int *size)
{
	int i;
	for(i=0;i<(*size);i++)
	{
		free((*thelist)[i].address);
		free((*thelist)[i].port);
	}
	if(*size) free(*thelist);
	*size=0;

	return 0;
}

int list_update_status(int index,int status,list_type **thelist,int *size)
{
	if(index>=(*size)) return 1;

	(*thelist)[index].status=status;

	return 0;
}
