/*
 *  PeaGnut - Command-line Gnutella client for Unix
 *  Copyright (C) 2000  Ciaran Lynch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/types.h>

#ifdef USE_SYSLOG
#include <syslog.h>
#else
#include <errno.h>
#include <string.h>
#endif

#include "log.h"

int log_open(void)
{
#ifdef USE_SYSLOG
	openlog(PACKAGE,LOG_PID,LOG_DAEMON);
#endif
	
	return 0;
}

int log(int priority,char *format,...)
{
	char print_string[1024];
	
	va_list ap;
	va_start(ap,format);
#if HAVE_VSNPRINTF
	vsnprintf(print_string,1024,format,ap);
#else
	vsprintf(print_string,format,ap); /* This is Bad - try to avoid */
#endif
	va_end(ap);
	
#ifdef USE_SYSLOG
	syslog(priority,print_string);
#else
	fprintf(stderr,"%s(%d) : %s\n",PACKAGE,priority,print_string);
	if(errno&&strstr(print_string,"%m"))
	{
		fprintf(stderr,"errno=%d : %s\n",errno,strerror(errno));
	}
#endif
	
	return 0;
}

int log_close(void)
{
#ifdef USE_SYSLOG
	closelog();
#endif
	
	return 0;
}
