/*
 *  PeaGnut - Command-line Gnutella client for Unix
 *  Copyright (C) 2000  Ciaran Lynch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>

#include "packetincoming.h"
#include "packetoutgoing.h"
#include "connectionlist.h"
#include "log.h"

int process_packet(list_type *connection,packet_type *packet)
{
	log(LOG_INFO,"Processing packet : type=%d from %s:%s",packet->type,connection->address,connection->port);
	switch(packet->type)
	{
		case 0x00 : /* Ping */
			outgoing_packet_write_pong(packet->guid,connection->address,connection->port);
			outgoing_packet_broadcast_ping(packet->guid,packet->ttl,packet->hops,connection->address,connection->port);
			break;

		case 0x01 : /* Pong */
			outgoing_packet_route_pong(packet->guid,packet->ttl,packet->hops,packet->payload_buffer,packet->payload_length,connection->address,connection->port);
			break;

		case 0x40 : /* Push Request */
			break;

		case 0x80 : /* Query */
			outgoing_packet_broadcast_query(packet->guid,packet->ttl,packet->hops,packet->payload_buffer,packet->payload_length,connection->address,connection->port);
			break;

		case 0x81 : /* Query Hits */
			outgoing_packet_route_queryhit(packet->guid,packet->ttl,packet->hops,packet->payload_buffer,packet->payload_length,connection->address,connection->port);
			break;

		default : /* Unknown packet, drop connection */
			return -1;
	}
	log(LOG_INFO,"Processing done");
	return 0;
}

int packet_incoming_process(void)
{
	/* Process completely received packets */

	int i;
	for(i=0;i<connection_list_size();i++)
	{
		list_type *connection=connection_list(i);
		if(connection->num_packets)
		{
			int j,break_loop=0;
			for(j=0;(!break_loop)&&(j<connection->num_packets);j++)
			{
				packet_type *packet=&(connection->packet[j]);
				if(packet->complete!=-1) continue;

				/* We have a complete packet, handle it ... */
				if(process_packet(connection,packet)==-1)
				{
					connection_list_remove(i);
					i--;
					break_loop=1;
				}
				/* Free packet now it has been processed */
				if(packet->payload_length&&packet->payload_buffer) free(packet->payload_buffer);
				if(connection->num_packets>1)
				{
					packet_type *new_packets=malloc(sizeof(packet_type)*((connection->num_packets)-1));
					int k;
					
					for(k=0;k<connection->num_packets;k++)
					{
						if(j!=k)
						{
							new_packets[(k<j)?k:k-1]=connection->packet[k];
						}
					}
					free(connection->packet);
					connection->packet=new_packets;
					(connection->num_packets)--;
				} else {
					free(connection->packet);
					connection->num_packets=0;
				}
			}
		}
	}
	
	return 0;
}

