/*
 *  PeaGnut - Command-line Gnutella client for Unix
 *  Copyright (C) 2000  Ciaran Lynch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <string.h>
#include <stdio.h> /* Why do you need a header for NULL ... */

#include "log.h"
#include "packetlist.h"

#define PACKET_LIST_SIZE_MAX 1000
packet_list_type *packet_list_internal=NULL;
int packet_list_size=0;

int packet_list_add(char *guid,char *address,char *port,int type)
{
	if(packet_list_size==PACKET_LIST_SIZE_MAX)
	{
		free(packet_list_internal[0].address);
		free(packet_list_internal[0].port);
		memcpy(packet_list_internal,packet_list_internal+1,sizeof(packet_list_type)*(packet_list_size-1)); /* Overlap ??? */
		packet_list_size--; /* So it will overwrite last entry */	
	} else {
		if(packet_list_size) {
			packet_list_type *new_packet_list=malloc(sizeof(packet_list_type)*(packet_list_size+1));
			if(new_packet_list==NULL)
			{
				log(LOG_ERR,"Unable to add packet from %s:%s to list : Out of memory",address,port);
				return 1;
			}
			memcpy(new_packet_list,packet_list_internal,sizeof(packet_list_type)*packet_list_size);
			free(packet_list_internal);
			packet_list_internal=new_packet_list;
		} else {
			packet_list_internal=malloc(sizeof(packet_list_type));
			if(packet_list_internal==NULL)
			{
				log(LOG_ERR,"Unable to add packet from %s:%s to list : Out of memory",address,port);
				return 1;
			}
		}
	}
	
	memcpy(packet_list_internal[packet_list_size].guid,guid,16);
	packet_list_internal[packet_list_size].address=strdup(address);
	packet_list_internal[packet_list_size].port=strdup(port);
	packet_list_internal[packet_list_size++].type=type;
	
	return 0;
}

int packet_list_search(char *guid)
{
	int i;
	for(i=0;i<packet_list_size;i++)
	{
		if(!memcmp(guid,packet_list_internal[i].guid,16))
		{
			/* Match found, return 1 */
			return 1;
		}
	}

	/* No match found, return 0 */
	return 0;
}

packet_list_type *packet_list(char *guid)
{
	int i;
	for(i=0;i<packet_list_size;i++)
	{
		if(!memcmp(guid,packet_list_internal[i].guid,16))
		{
			/* Match found, return */
			packet_list_type *new_packet=malloc(sizeof(packet_list_type));
			if(new_packet==NULL)
			{
				log(LOG_ERR,"Unable to return packet from %s:%s : Out of memory",packet_list_internal[i].address,packet_list_internal[i].port);
				return NULL;
			}
			memcpy(new_packet->guid,packet_list_internal[i].guid,16);
			new_packet->address=strdup(packet_list_internal[i].address);
			new_packet->port=strdup(packet_list_internal[i].port);
			new_packet->type=packet_list_internal[i].type;

			return new_packet;
		}
	}
	
	/* No match found, return NULL */
	return NULL;
}

int packet_list_free(void)
{
	int i;
	for(i=0;i<packet_list_size;i++)
	{
		free(packet_list_internal[i].address);
		free(packet_list_internal[i].port);
	}
	if(packet_list_size) free(packet_list_internal);
	packet_list_size=0;
	
	return 0;
}
