/*
 *  PeaGnut - Command-line Gnutella client for Unix
 *  Copyright (C) 2000  Ciaran Lynch
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <time.h>
#include <stdlib.h>

#include <sys/socket.h>

#include "packetoutgoing.h"
#include "completeio.h"
#include "log.h"
#include "packetlist.h"
#include "connectionlist.h"
#include "server.h"

#define NEW_TTL 5
/*
int write_close_check(void *buffer,int size,list_type *list_item,int con)
{
	int read_ret;

	if((read_ret=write_all(list_item->con_socket,buffer,size))==-1)
	{
		return -1; / Error /
	}
	if(!read_ret)
	{
		/ Most likely place for it /
		log(LOG_INFO,"Outgoing connection removed : %s:%s",list_item->address,list_item->port);
		connection_list_remove(con);
		free(list_item->address);
		free(list_item->port);
		free(list_item);
		return 1;
	}

	return 0;
}*/

int write_header(char *guid,int type,int ttl,int hops,int payload_size,list_type *list_item)
{
	int retval;
	retval=write_all(list_item->con_socket,guid,16);
	if(retval==-1) return -1;
	else if(retval==0) return 0;
	retval=write_all(list_item->con_socket,&type,sizeof(type));
	if(retval==-1) return -1;
	else if(retval==0) return 0;
	retval=write_all(list_item->con_socket,&ttl,sizeof(ttl));
	if(retval==-1) return -1;
	else if(retval==0) return 0;
	retval=write_all(list_item->con_socket,&hops,sizeof(hops));
	if(retval==-1) return -1;
	else if(retval==0) return 0;
	retval=write_all(list_item->con_socket,&payload_size,sizeof(payload_size));
	if(retval==-1) return -1;
	else if(retval==0) return 0;

	return 1;
}

int write_new_header(char *guid,int type,int payload_size,list_type *list_item)
{
	return write_header(guid,type,NEW_TTL,0,payload_size,list_item); 
}

int outgoing_packet_write_pong(char *guid,char *address,char *port)
{
	/* Write a PONG reply from this host */
	int i,retval;
	
	for(i=0;i<connection_list_size();i++)
	{
		list_type *list_item=connection_list(i);
		if(list_item==NULL)
		{
			log(LOG_ERR,"Unable to write PONG packet : Error reading list item");
			return 1;
		}
		
		if(!strcmp(address,list_item->address) && !strcmp(port,list_item->port))
		{
			unsigned char payload[14];
			/* Write to this host ... */
			
			/* Header */
			retval=write_new_header(guid,0x01,14,list_item);
			if(retval==-1) {
				log(LOG_ERR,"Unable to write PONG packet to %s:%s : Error writing to socket : %m");
				return 1;
			} else if(retval==0) {
				log(LOG_ERR,"Unable to write PONG packet to %s:%s : Connection closed");
				return 1;
			}
			
			/* Payload */
			payload[0]=(unsigned char)((main_server_port&0xff00)>>8);
			payload[1]=(unsigned char)(main_server_port&0x00ff);
			payload[2]=(unsigned char)((main_server_ip&0x0000ff00)>>8);
			payload[3]=(unsigned char)(main_server_ip&0x000000ff);
			payload[4]=(unsigned char)((main_server_ip&0xff000000)>>24);
			payload[5]=(unsigned char)((main_server_ip&0x00ff0000)>>16);
			payload[6]=payload[7]=payload[8]=payload[9]=0; /* Files */
			payload[10]=payload[11]=payload[12]=payload[13]=0; /* Kb */
			retval=write_all(list_item->con_socket,payload,14);
			if(retval==-1) {
				log(LOG_ERR,"Unable to write PONG packet to %s:%s : Error writing to socket : %m");
				return 1;
				
			} else if(retval==0) {
				log(LOG_ERR,"Unable to write PONG packet to %s:%s : Connection closed");
				return 1;
			}
			log(LOG_INFO,"PONG Written to host %s:%s",list_item->address,list_item->port);
		}
	}

	return 0;
}

int outgoing_packet_route_pong(char *guid,int ttl,int hops,void *payload,int payload_length,char *address,char *port)
{
	int retval;
	/* Assert: There is a packet with the same guid in the list [PING] */
	packet_list_type *ping_packet=packet_list(guid);
	if(!ping_packet)
	{
		log(LOG_ERR,"Unable to route PONG packet : Unable to find corresponding PING");
		return 1;	
	} else {
		int i;
		for(i=0;i<connection_list_size();i++)
		{		
			list_type *list_item=connection_list(i);
			if(list_item==NULL)
			{
				log(LOG_ERR,"Unable to route PONG packet : Error returning list item");
				return 1;
			}
			
			if(!strcmp(ping_packet->address,list_item->address) && !strcmp(ping_packet->port,list_item->port))
			{
				/* Found the right one ... */
				retval=write_header(guid,0x01,ttl-1,hops+1,payload_length,list_item);
				if(retval==-1)
				{
					log(LOG_ERR,"Unable to route PONG packet to %s:%s : Error writing to socket : %m",list_item->address,list_item->port);
					free(ping_packet->address);
					free(ping_packet->port);
					free(ping_packet);
					return 1;
				} else if(retval==0) {
					log(LOG_ERR,"Unable to route PONG packet to %s:%s : Connection closed",list_item->address,list_item->port);
					free(ping_packet->address);
					free(ping_packet->port);
					free(ping_packet);
					return 1;
				}
				retval=write_all(list_item->con_socket,payload,payload_length);
				if(retval==-1) {
					log(LOG_ERR,"Unable to route PONG packet to %s:%s : Error writing to socket : %m",list_item->address,list_item->port); 
					free(ping_packet->address);
					free(ping_packet->port);
					free(ping_packet);
					return 1;
				} else if(retval==0) {
					log(LOG_ERR,"Unable to route PONG packet to %s:%s : Connection closed",list_item->address,list_item->port); 
					free(ping_packet->address);
					free(ping_packet->port);
					free(ping_packet);
					return 1;
				}
				log(LOG_INFO,"PONG Forwarded to %s:%s from %s:%s",list_item->address,list_item->port,address,port);
			}
		}
	}

	free(ping_packet->address);
	free(ping_packet->port);
	free(ping_packet);
	
	return 0;
}

int outgoing_packet_broadcast_ping(char *guid,int ttl,int hops,char *address,char *port)
{
	int i,retval;
	for(i=0;i<connection_list_size();i++)
	{
		list_type *list_item=connection_list(i);
		if(list_item==NULL)
		{
			log(LOG_ERR,"Unable to broadcast ping from %s:%s : Unable to return list item",address,port);
			return 1;
		}
		
		if(strcmp(address,list_item->address)||strcmp(port,list_item->port))
		{
			/* Not originating host */
			retval=write_header(guid,0x00,ttl-1,hops+1,0,list_item);
			if(retval==-1) {
				/* Close connection  */
				log(LOG_ERR,"Unable to broadcast ping to %s:%s : Error writing socket : %m",list_item->address,list_item->port);
				if(shutdown(list_item->con_socket,2)==-1)
					log(LOG_ERR,"Unable to shutdown socket : %m");
				if(close(list_item->con_socket)==-1)
					log(LOG_ERR,"Unable to close socket : %m");
				connection_list_remove(i);
				i--;
				continue;
			} else if(retval==0) {
				log(LOG_ERR,"Unable to broadcast ping to %s:%s : Connection closed",list_item->address,list_item->port);
				if(shutdown(list_item->con_socket,2)==-1)
					log(LOG_ERR,"Unable to shutdown socket : %m");
				if(close(list_item->con_socket)==-1)
					log(LOG_ERR,"Unable to close socket : %m");
				connection_list_remove(i);
				i--;
				continue;
			}
			/* PINGs have no payload ... */
			log(LOG_INFO,"PING Broadcast to %s:%s from %s:%s",list_item->address,list_item->port,address,port);
		}
	}
	
	return 0;
}

int generate_guid(char *buffer)
{
	int i;
	static int seeded=0;

	if(!seeded)
	{
		srand(time(NULL));
	}
	for(i=0;i<16;i++) buffer[i]=(unsigned char)(rand()%0xff);

	return 0;
}

int outgoing_packet_write_ping(char *address,char *port)
{
	/* Write a PING from this host */
	int i,retval;
	
	for(i=0;i<connection_list_size();i++)
	{
		list_type *list_item=connection_list(i);
		if(list_item==NULL)
		{
			log(LOG_ERR,"Unable to write PING packet to %s:%s : Error reading from connection list",address,port);
			return 1;
		}
		
		if(!strcmp(address,list_item->address) && !strcmp(port,list_item->port))
		{
			char guid_buffer[16];
			/* Write to this host ... */
			
			generate_guid(guid_buffer);
			/* Header */
			retval=write_new_header(guid_buffer,0x00,0,list_item);
			if(retval==-1) {
				log(LOG_ERR,"Unable to write PING packet to %s:%s : Error writing to socket : %m",list_item->address,list_item->port);
				
				return 1;
			} else if(retval==0) {
				log(LOG_ERR,"Unable to write PING packet to %s:%s : Connection closed",list_item->address,list_item->port);
				
				return 1;
			}
			
			log(LOG_INFO,"PING Written to host %s:%s",list_item->address,list_item->port);
		}
	}

	return 0;
}

int outgoing_packet_broadcast_query(char *guid,int ttl,int hops,void *payload,int payload_length,char *address,char *port)
{
	int i,retval;
	for(i=0;i<connection_list_size();i++)
	{
		list_type *list_item=connection_list(i);
		if(list_item==NULL)
		{
			log(LOG_ERR,"Unable to broadcast query from %s:%s : Unable to return list item",address,port);
			return 1;
		}
		
		if(strcmp(address,list_item->address)||strcmp(port,list_item->port))
		{
			/* Not originating host */
			retval=write_header(guid,0x00,ttl-1,hops+1,payload_length,list_item);
			if(retval==-1) {
				/* Close connection  */
				log(LOG_ERR,"Unable to broadcast query to %s:%s : Error writing socket : %m",list_item->address,list_item->port);
				if(shutdown(list_item->con_socket,2)==-1)
					log(LOG_ERR,"Unable to shutdown socket : %m");
				if(close(list_item->con_socket)==-1)
					log(LOG_ERR,"Unable to close socket : %m");
				connection_list_remove(i);
				i--;
				continue;
			} else if(retval==0) {
				log(LOG_ERR,"Unable to broadcast query to %s:%s : Connection closed",list_item->address,list_item->port);
				if(shutdown(list_item->con_socket,2)==-1)
					log(LOG_ERR,"Unable to shutdown socket : %m");
				if(close(list_item->con_socket)==-1)
					log(LOG_ERR,"Unable to close socket : %m");
				connection_list_remove(i);
				i--;
				continue;
			}
			/* QUERYs do have a payload ... */
			retval=write_all(list_item->con_socket,payload,payload_length);
			if(retval==-1) {
				/* Close connection  */
				log(LOG_ERR,"Unable to broadcast query to %s:%s : Error writing socket : %m",list_item->address,list_item->port);
				if(shutdown(list_item->con_socket,2)==-1)
					log(LOG_ERR,"Unable to shutdown socket : %m");
				if(close(list_item->con_socket)==-1)
					log(LOG_ERR,"Unable to close socket : %m");
				connection_list_remove(i);
				i--;
				continue;
			} else if(retval==0) {
				log(LOG_ERR,"Unable to broadcast query to %s:%s : Connection closed",list_item->address,list_item->port);
				if(shutdown(list_item->con_socket,2)==-1)
					log(LOG_ERR,"Unable to shutdown socket : %m");
				if(close(list_item->con_socket)==-1)
					log(LOG_ERR,"Unable to close socket : %m");
				connection_list_remove(i);
				i--;
				continue;
			}
			
			log(LOG_INFO,"QUERY Broadcast to %s:%s from %s:%s",list_item->address,list_item->port,address,port);
		}
	}
	
	return 0;
}

int outgoing_packet_route_queryhit(char *guid,int ttl,int hops,void *payload,int payload_length,char *address,char *port)
{
	int retval;
	/* Assert: There is a packet with the same guid in the list [QUERY] */
	packet_list_type *query_packet=packet_list(guid);
	if(!query_packet)
	{
		log(LOG_ERR,"Unable to route QUERYHIT packet : Unable to find corresponding QUERY");
		return 1;	
	} else {
		int i;
		for(i=0;i<connection_list_size();i++)
		{		
			list_type *list_item=connection_list(i);
			if(list_item==NULL)
			{
				log(LOG_ERR,"Unable to route QUERYHIT packet : Error returning list item");
				return 1;
			}
			
			if(!strcmp(query_packet->address,list_item->address) && !strcmp(query_packet->port,list_item->port))
			{
				/* Found the right one ... */
				retval=write_header(guid,0x01,ttl-1,hops+1,payload_length,list_item);
				if(retval==-1)
				{
					log(LOG_ERR,"Unable to route QUERYHIT packet to %s:%s : Error writing to socket : %m",list_item->address,list_item->port);
					free(query_packet->address);
					free(query_packet->port);
					free(query_packet);
					return 1;
				} else if(retval==0) {
					log(LOG_ERR,"Unable to route QUERYHIT packet to %s:%s : Connection closed",list_item->address,list_item->port);
					free(query_packet->address);
					free(query_packet->port);
					free(query_packet);
					return 1;
				}
				retval=write_all(list_item->con_socket,payload,payload_length);
				if(retval==-1) {
					log(LOG_ERR,"Unable to route QUERYHIT packet to %s:%s : Error writing to socket : %m",list_item->address,list_item->port); 
					free(query_packet->address);
					free(query_packet->port);
					free(query_packet);
					return 1;
				} else if(retval==0) {
					log(LOG_ERR,"Unable to route QUERYHIT packet to %s:%s : Connection closed",list_item->address,list_item->port); 
					free(query_packet->address);
					free(query_packet->port);
					free(query_packet);
					return 1;
				}
				log(LOG_INFO,"QUERYHIT Forwarded to %s:%s from %s:%s",list_item->address,list_item->port,address,port);
			}
		}
	}

	free(query_packet->address);
	free(query_packet->port);
	free(query_packet);
	
	return 0;
}

